<?php
namespace Teknisa\CTB\Services;

use Teknisa\CTB\Services\Ctb23000BalanceteGerencialFactory;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;

class Ctb23000BalanceteGerencialService extends \Teknisa\CTB\Controllers\TeknisaCrud {

    protected $balanceteGerencialFactory;

    public function __construct($balanceteGerencialFactory, $entityManager) {
        $this->balanceteGerencialFactory = $balanceteGerencialFactory;
        $this->connection  = $entityManager->getConnection();
    }

    public function buscaDadosBalancete($row, $page) {
        $cdFilialConsul     = $this->montaSqlFilial($row['CDFILIAL']);
        $cdFilialGera   = $this->montaInStrField($row['CDFILIAL']);
        $cdEmpresa = is_array($row['CDEMPRESA']) ? implode('_',$row['CDEMPRESA']) : $row['CDEMPRESA'];
        $cdCentCustGera   = $this->montaInStrField(isset($row['CDCENTCUST']) ? $row['CDCENTCUST'] : $this->getAllCdCentCust($cdFilialConsul));
        $cdCentCustConsul = $this->montaSqlCentCust(isset($row['CDCENTCUST']) ? $row['CDCENTCUST'] : $this->getAllCdCentCust($cdFilialConsul));
        $dataIni    = (string) date('d/m/Y', strtotime(str_replace('/', '-', $row['DATABALANCETE'][0])));
        $dataFin    = (string) date('d/m/Y', strtotime(str_replace('/', '-', $row['DATABALANCETE'][1])));
        $dataSaldo  = (string) $this->getDtSaldo($dataIni);
        $numDigCont = strlen(str_replace('.', '', $this->balanceteGerencialFactory->getParamGeral()['CDPICTCCTB']));

        $this->getCamposConta($row['CDCONTFIN'], $numDigCont);

        $contaIni   = $row['CDCONTINI'];
        $contaFin   = $row['CDCONTFIN'];
        $cdNvCont   = $row['CDNVCONT'];

        if($page == 1) {
            try {
                $this->connection->beginTransaction();
                $nrBalancete = $this->geraDadosBalancete($cdFilialGera, $cdEmpresa, $cdCentCustGera,
                $dataIni, $dataFin, $dataSaldo, $contaIni, $contaFin, $cdNvCont);
                $this->connection->commit();
            }
            catch (\Exception $e) {
                $this->connection->rollback();
                throw $e;
            }
            $minCont = 'T';
        }
        else {
            $nrBalancete = $row['NRCTBBALGERTMP'];
            $minCont = $row['MAXCONTCTBL'];
        }

        return $this->balanceteGerencialFactory->buscaDadosBalancete($cdCentCustConsul, $nrBalancete, $minCont);
    }

    public function getAllDistinctCdCentCust($cdFilial, $centCusts) {
        $cdFilial = $this->montaSqlFilial($cdFilial);
        if($centCusts == null){
            $centCust = "'T'";
        } else {
            $centCust = $this->montaSqlCentCust($centCusts);
        }
        return $this->balanceteGerencialFactory->getAllCdCentCust($cdFilial, $centCust);
    }

    private function geraDadosBalancete($cdFilial, $cdEmpresa, $cdCentCustGera,
            $dataIni, $dataFin, $dataSaldo, $contaIni, $contaFin, $cdNvCont) {
        return $this->balanceteGerencialFactory->geraDadosBalancete($cdFilial, $cdEmpresa, $cdCentCustGera,
            $dataIni, $dataFin, $dataSaldo, $contaIni, $contaFin, $cdNvCont);
    }

    private function getCamposConta(&$contFin, $numDigCont) {
        $contFin = (strlen($contFin) < $numDigCont) ? str_pad($contFin, $numDigCont, "z", STR_PAD_RIGHT) : substr($contFin, 0, $numDigCont);
    }


    private function montaInStrField($valueField) {
        $sql = '_';

        foreach($valueField as $value) {
            $value = (isset($value["CDCENTCUST"])) ? $value["CDCENTCUST"] : $value;
            $sql.= $value.'_';
        }

        return $sql;
    }

    private function montaSqlFilial($cdFilial) {
        $sql = '(';
        if($cdFilial) {
            foreach($cdFilial as $value) {
                $sql.= "'".$value."'".',';
            }
        }
        else throw new \Exception("Não foi encontrado nenhum centro de custo.");
        $size = strlen($sql);
        return substr($sql,0, $size-1).')';
    }

    private function montaSqlCentCust($cdCentCust) {
        $sql = '(';
        if($cdCentCust) {
            foreach($cdCentCust as $value) {
                $value = (isset($value["CDCENTCUST"])) ? $value["CDCENTCUST"] : $value;
                $sql.= "'".$value."'".',';
            }
        }
        else throw new \Exception("Não foi encontrado nenhum centro de custo.");
        $size = strlen($sql);
        return substr($sql,0, $size-1).')';
    }


    // Retorna o ultimo dia do mês anteriror a dataInicial
    private function getDtSaldo($dataInicial) {
        $dataInicial = str_replace('/', '-', $dataInicial);
        $date = new \DateTime($dataInicial);
        $date->modify('-1 month');
        $date->modify('last day of this month');
        return $date->format('d/m/Y'); //dia mês e ano
    }

    private function getAllCdCentCust($cdFilial) {
        return $this->balanceteGerencialFactory->getAllCdCentCust($cdFilial);
    }

}